diff --git a/arch/arm/mach-k3/j721e_init.c b/arch/arm/mach-k3/j721e_init.c
index f1ce45d69b..07be1f0bb7 100644
--- a/arch/arm/mach-k3/j721e_init.c
+++ b/arch/arm/mach-k3/j721e_init.c
@@ -25,6 +25,10 @@
 #include <dm/root.h>
 #include "../../../board/ti/common/board_detect.h"
 
+#define GPU_HARDWARE_BASE_ADDRESS_READ  0x45DC5100
+#define GPU_HARDWARE_BASE_ADDRESS_WRITE 0x45DC5900
+#define REGISTER_VALUE_DISABLE_SNOOP    0x30000000
+
 #ifdef CONFIG_SPL_BUILD
 #ifdef CONFIG_K3_LOAD_SYSFW
 struct fwl_data cbass_hc_cfg0_fwls[] = {
@@ -100,6 +104,19 @@ static void ctrl_mmr_unlock(void)
 	mmr_unlock(CTRL_MMR0_BASE, 7);
 }
 
+static inline void _qos_work_around_for_gpu(phys_addr_t base)
+{
+	u32 offset;
+
+	for (offset = 0; offset < 8; offset++)
+		writel(REGISTER_VALUE_DISABLE_SNOOP, base + (offset * 4));
+}
+
+static void qos_work_around_for_gpu(void)
+{
+	_qos_work_around_for_gpu(GPU_HARDWARE_BASE_ADDRESS_READ);
+}
+
 #if defined(CONFIG_K3_LOAD_SYSFW)
 void k3_mmc_stop_clock(void)
 {
@@ -330,6 +347,19 @@ void board_init_f(ulong dummy)
 	if (ret)
 		panic("DRAM init failed: %d\n", ret);
 #endif
+
+	if (soc_is_j721e()) {
+		setup_navss_nb();
+		setup_c66_qos();
+		setup_main_r5f_qos();
+		setup_vpac_qos();
+		setup_dmpac_qos();
+		setup_dss_qos();
+		setup_gpu_qos();
+		setup_encoder_qos();
+		qos_work_around_for_gpu();
+	}
+
 	spl_enable_dcache();
 }
 
